@@ -122,3 +122,27 @@ test_that("TABU Search metaheuristic results", {
 })
 
 
+test_that("GA Search metaheuristic results", {
+  skip_on_cran()
+
+  level<- elog.level()
+  elog.level("ERROR")
+
+  set.seed(2718282)
+  f<- PlainFunction$new(f0.cigar4)
+  f$Parameter(name="x1",min=-100,max=100)
+  f$Parameter(name="x2",min=-100,max=100)
+  f$Parameter(name="x3",min=-100,max=100)
+  f$Parameter(name="x4",min=-100,max=100)
+  v<- extremize("ga", f)
+  results<- v$getBest()
+
+  expect_equal(round(results$x1,3), 0)
+  expect_equal(round(results$x2,3), 0)
+  expect_equal(round(results$x3,3), 0)
+  expect_equal(round(results$x4,3), 0)
+
+  elog.level(level)
+})
+
+
